/*
 * HwBuilder - An utility for building Harbour applications
 * Main file
 *
 * Copyright 2002 Alexander S.Kresin <alex@belacy.belgorod.su>
 * www - http://www.geocities.com/alkresin/
*/

#include "windows.ch"
#include "guilib.ch"
#include "hwbuild.h"

Function Main()
Local oMainWindow, hDCwindow, oIcon := HIcon():AddResource("ICON_1")
PRIVATE aTermMetr := { 800 }
PRIVATE oProject := HProject():New()
PRIVATE lChgSettings := .F., cDirHrb
PRIVATE mypath := "\" + CURDIR() + IIF( EMPTY( CURDIR() ), "", "\" )

   INIT WINDOW oMainWindow MAIN ICON oIcon TITLE "HwBuilder" ;
         ON SIZE {|o|MoveWindow(o:handle,0,0,aTermMetr[1],100)} ;
         ON EXIT {||CloseProject()}

   MENU OF oMainWindow
      MENU TITLE "File"
         MENUITEM "New Project" ACTION NewProject()
         MENUITEM "Open Project" ACTION OpenProject()
         MENUITEM "Save Project" ACTION SaveProject()
         SEPARATOR
         MENUITEM "Exit" ACTION EndWindow()
      ENDMENU
      MENU TITLE "Project"
         MENUITEM "Add to Project" ACTION AddFile()
         MENUITEM "Remove from Project" ACTION RemoveFile()
         SEPARATOR
         MENUITEM "Create command file" ACTION CreateBat()
         MENUITEM "Build" ACTION MsgInfo("Build")
         SEPARATOR
         MENUITEM "Options" ACTION Options()
      ENDMENU
      MENU TITLE "Global Options"
         MENUITEM "Directories" ACTION Directories()
      ENDMENU
      MENUITEM "About" ACTION MsgInfo("HwBuilder 1.0","About")
   ENDMENU

   hDCwindow := GetDC( oMainWindow:handle )
   aTermMetr := GetDeviceArea( hDCwindow )
   DeleteDC( hDCwindow )

   ReadSettings()

   ACTIVATE WINDOW oMainWindow

   IF lChgSettings
      WriteSettings()
   ENDIF

Return

Static Function ReadSettings
Local hkey1 := RegOpenKey( HKEY_CURRENT_USER, "Software" ), hkey2
Local cRegSettings
   if hkey1 != -1
      hkey2 := RegOpenKey( hkey1, "AlKresin_Soft" )
      if hkey2 != -1
         cRegSettings := RegGetValue( hkey2,"GlobalSettings" )
         RegCloseKey( hkey2 )
      endif
      RegCloseKey( hkey1 )
   endif
   if cRegSettings == Nil
     cDirHrb := "C:\Harbour"
   else
     cDirHrb := cRegSettings
   endif
Return Nil

Static Function WriteSettings
Local hkey1 := RegOpenKey( HKEY_CURRENT_USER, "Software" ), hkey2
Local cRegSettings
   if hkey1 != -1
      hkey2 := RegCreateKey( hkey1, "AlKresin_Soft" )
      if hkey2 != -1
         RegSetString( hkey2,"GlobalSettings",cDirHrb )
         RegCloseKey( hkey2 )
      endif
      RegCloseKey( hkey1 )
   endif
Return Nil

Static Function NewProject()
   oProject := HProject():New()
Return Nil

Static Function OpenProject()
Local fname := SelectFile( "Harbour project files( *.prj )", "*.prj", mypath )
Private aFiles := {}, aLibraries := {}
Private nType := 1,lAuto := .F., lImplicit := .T., lDebug := .F., lSupress := .F.
Private lPpo := .F., lMemvars := .F.

   IF Empty( fname )
      Return Nil
   ENDIF

   CloseProject()
   mypath := "\" + CURDIR() + IIF( EMPTY( CURDIR() ), "", "\" )
   IF RdIni( fname ) != 0
   ENDIF

Return Nil

Static Function CloseProject()
   IF oProject:lChanged
      IF MsgYesNo( "Project was changed. Are you want to save it ?" )
         SaveProject()
      ENDIF
   ENDIF
Return .T.

Static Function SaveProject()
Local fname := SaveFile( "*.prj","Harbour project files( *.prj )","*.prj",mypath )
Local han, i, kolfiles := Len( oProject:aFiles ), kollibr := Len( oProject:aLibraries )

   IF !Empty( fname )
      han := Fcreate( fname )
      IF kolfiles > 0
         Fwrite( han,"aFiles={ &"+Chr(10) )
         FOR i := 1 TO kolfiles
            Fwrite( han, '"'+aFiles[i]+'"'+Iif(i==kolfiles," }",", &" )+Chr(10) )
         NEXT
      ENDIF
      IF kollibr > 0
         Fwrite( han,"aLibraries={ &"+Chr(10) )
         FOR i := 1 TO kollibr
            Fwrite( han, '"'+aLibraries[i]+'"'+Iif(i==kollibr," }",", &" )+Chr(10) )
         NEXT
      ENDIF
      Fwrite( han, "nType="+Str(oProject:nType,1)+Chr(10) )
      Fwrite( han, "lAuto="+Iif(oProject:lAuto,"On","Off")+Chr(10) )
      Fwrite( han, "lImplicit="+Iif(oProject:lImplicit,"On","Off")+Chr(10) )
      Fwrite( han, "lDebug="+Iif(oProject:lDebug,"On","Off")+Chr(10) )
      Fwrite( han, "lSupress="+Iif(oProject:lSupress,"On","Off")+Chr(10) )
      Fwrite( han, "lPpo="+Iif(oProject:lPpo,"On","Off")+Chr(10) )
      Fwrite( han, "lMemvars="+Iif(oProject:lMemvars,"On","Off")+Chr(10) )
      Fclose( han )
   ENDIF 

Return Nil

Static Function AddFile()
Local mypath := "\" + CURDIR() + IIF( EMPTY( CURDIR() ), "", "\" )
Local fname := SelectFile( "Harbour source files( *.prg )", "*.prg", mypath )

   IF !Empty( fname )
      mypath := "\" + CURDIR() + IIF( EMPTY( CURDIR() ), "", "\" )
      Aadd( oProject:aFiles, CutPath( fname ) )
   ENDIF

Return Nil

Function RemoveFile
Local oModDlg
Private lOk

   IF Len( oProject:aFiles ) > 0

      INIT DIALOG oModDlg FROM RESOURCE TITLE "REMOVE_DLG"

      REDEFINE BROWSE oBrw ARRAY OF oModDlg ID ID_BRW    ;
          ON CLICK {|| lOk := .T., EndDialog( getmodalhandle() )}

      oBrw:bcolorSel := VColor( "C80000" )
      oBrw:lDispHead := .F.
      CreateArList( oBrw, oProject:aFiles )
      oBrw:aColumns[1]:length := 15

      ACTIVATE DIALOG oModDlg

      IF oModDlg:lResult
         Adel( oProject:aFiles, oBrw:tekzp )
         Asize( oProject:aFiles, Len(oProject:aFiles)-1 )
      ENDIF
   ELSE
      MsgInfo( "No files in project","Remove file" )
   ENDIF
Return Nil

Static Function CreateBat()
Local han := Fcreate( oProject:name+".bat" )
Local i, kolfiles := Len( oProject:aFiles ), cOpt := ""

   Fwrite( han,"SET HARB_INSTALL="+cDirHrb+Chr(10) )
   IF oProject:lAuto
      cOpt += "-a"
   ENDIF
   IF oProject:lImplicit
      cOpt += " -n"
   ENDIF
   IF oProject:lDebug
      cOpt += " -b"
   ENDIF
   IF oProject:lSupress
      cOpt += " -l"
   ENDIF
   IF oProject:lPpo
      cOpt += " -p"
   ENDIF
   IF oProject:lMemvars
      cOpt += " -v"
   ENDIF
   FOR i := 1 TO kolfiles
      Fwrite( han,"%HARB_INSTALL%\bin\harbour.exe "+cOpt+aFiles[i]+ ;
                Iif(i==1," >"," >>")+oProject:name+".out"+Chr(10) )
   NEXT
   FOR i := 1 TO kolfiles
      Fwrite( han,"bcc32 -c -I%HARB_INSTALL%\include"+Cutexten(aFiles[i])+ ;
                ".c"+" >>"+oProject:name+".out"+Chr(10) )
   NEXT
   Fwrite( han, "ilink32 "+Chr(10) )
   Fclose( han )

Return Nil

STATIC FUNCTION Options()
Local oDlg1, oDlg2, oDlg3
Local lAuto := oProject:lAuto, lImplicit := oProject:lImplicit,  ;
      lDebug := oProject:lDebug, lSupress := oProject:lSupress,  ;
      lPpo := oProject:lPpo, lMemvars := oProject:lMemvars, nType := oProject:nType

   INIT DIALOG oDlg1 FROM RESOURCE TITLE "PAGE_1"
     GET RADIOGROUP nType
     REDEFINE RADIOBUTTON OF oDlg1 ID IDC_RADIOBUTTON1
     REDEFINE RADIOBUTTON OF oDlg1 ID IDC_RADIOBUTTON2
     REDEFINE RADIOBUTTON OF oDlg1 ID IDC_RADIOBUTTON3
     END RADIOGROUP
   INIT DIALOG oDlg2 FROM RESOURCE TITLE "PAGE_2"
      REDEFINE GET CHECKBOX VAR lAuto     OF oDlg2 ID IDC_CHECKBOX1 COLOR Vcolor("0000FF")
      REDEFINE GET CHECKBOX VAR lImplicit OF oDlg2 ID IDC_CHECKBOX2
      REDEFINE GET CHECKBOX VAR lDebug    OF oDlg2 ID IDC_CHECKBOX3
      REDEFINE GET CHECKBOX VAR lSupress  OF oDlg2 ID IDC_CHECKBOX4
      REDEFINE GET CHECKBOX VAR lPpo      OF oDlg2 ID IDC_CHECKBOX5
      REDEFINE GET CHECKBOX VAR lMemvars  OF oDlg2 ID IDC_CHECKBOX6
   INIT DIALOG oDlg3 FROM RESOURCE TITLE "PAGE_3"

   PropertySheet( GetActiveWindow(),{ oDlg1,oDlg2,oDlg3 }, "Project Options" )

   IF oDlg2:lResult != Nil .AND. oDlg2:lResult
      oProject:nType     := nType
      oProject:lAuto     := lAuto
      oProject:lImplicit := lImplicit
      oProject:lDebug    := lDebug
      oProject:lSupress  := lSupress
      oProject:lPpo      := lPpo
      oProject:lMemvars  := lMemvars
   ENDIF

RETURN Nil

STATIC FUNCTION Directories()
Local oDlg
Local oFont := HFont():Add( "MS Sans Serif",0,-13 )

   INIT DIALOG oModDlg TITLE "Directories" ;
        AT 210,20 SIZE 300,140             ;
        FONT oFont

   @ 20,10 SAY "Harbour directory" OF oModDlg ID 101 ;
        SIZE 260,22 STYLE WS_VISIBLE + WS_CHILD
   @ 20,35 EDITBOX cDirHrb OF oModDlg ID 102 ;
            SIZE 260,26                 ;
            STYLE WS_DLGFRAME + WS_VISIBLE + WS_CHILD + WS_TABSTOP
   @ 20,95 BUTTON "Ok" OF oModDlg ID IDOK ;
            SIZE 100,32                   ;
            ON CLICK {|o|EndDirDlg(o)}
   @ 180,95 BUTTON "Cancel" OF oModDlg ID IDCANCEL ;
            SIZE 100,32

   ACTIVATE DIALOG oModDlg

RETURN Nil

Static Function EndDirDlg( oDlg )
Local cdh := GetEditText( oDlg:handle,102 )
   IF cdh != cDirHrb
      cDirHrb := cdh
      lChgSettings := .T.
   ENDIF
   EndDialog( oDlg:handle )
RETURN Nil

FUNCTION CutPath( fname )
LOCAL i := RAT( '\', fname )
   IF i == 0
      i := RAT( '/', fname )
   ENDIF
RETURN IIF( i = 0, fname, SUBSTR( fname, i + 1 ) )